const formFields = [
    'input',
    'select',
    'textarea'
].map(el => el.toLowerCase());

export class FormBuilder {
    constructor(formName, fields) {
        this.name = formName;
        this.value = {};
        this.validators = {};

        for (let fieldName in fields) {
            const field = fields[fieldName];

            if (field.length === 0) {
                this.value[fieldName] = null;
            } else {
                this.value[fieldName] = field[0];
            }

            if (field.length === 2) {
                if (field[1] instanceof Array) {
                    this.validators[fieldName] = field[1];
                } else {
                    this.validators[fieldName] = [field[1]];
                }
            } else {
                this.validators[fieldName] = [];
            }
        }

        const formElement = document.querySelector(`form[name="${formName}"]`);

        for (let fieldName in this.value) {
            let fieldElement = formElement.querySelector(`[name=${fieldName}]`);

            if (fieldElement !== null) {
                if (formFields.includes(fieldElement.tagName.toLowerCase())) {
                    fieldElement.value = this.value[fieldName];
                    fieldElement.addEventListener('input', () => {
                        this.value[fieldName] = fieldElement.value;
                    });
                }
            }
        }
    }

    validate() {
        let result = true;

        for (let fieldName in this.value) {
            const validators = this.validators[fieldName];

            for (let i = 0; i < validators.length; i++) {
                if (!validators[i](this.value[fieldName], this)) {
                    result = false;
                    break;
                }
            }

            if (!result) {
                break;
            }
        }

        return result;
    }
}

export class Validators {
    static required(val) {
        return val !== null && val !== '';
    }
    static pattern(patt) {
        return (val) => {
            const isNotEmpty = this.required(val);

            if (!isNotEmpty) {
                return false;
            }

            const validator = new RegExp(patt);
            return validator.test(val);
        };
    }
    static match(fieldName) {
        return (val, that) => {
            return val === that.value[fieldName];
        };
    }
}